@@ -1,54 +1,71 @@
 import csv
-import io
-import itertools
+import datetime
 import functools
+import io
 import tempfile
 from typing import List, Optional
 
-import dataclasses
-
 import falcon
+import orjson
+import peewee
 import requests
+from falcon.media.validators import jsonschema
 from falcon_cors import CORS
 from falcon_multipart.middleware import MultipartMiddleware
-
 from PIL import Image
+from sentry_sdk.integrations.falcon import FalconIntegration
 
 from robotoff import settings
+from robotoff.app import schema
+from robotoff.app.auth import BasicAuthDecodeError, basic_decode
 from robotoff.app.core import get_insights, save_insight
-from robotoff.app.auth import basic_decode, BasicAuthDecodeError
 from robotoff.app.middleware import DBConnectionMiddleware
-from robotoff.ingredients import generate_corrections, generate_corrected_text
-from robotoff.insights._enum import InsightType
-from robotoff.insights.extraction import extract_ocr_insights, DEFAULT_INSIGHT_TYPES
-from robotoff.insights.ocr.dataclass import OCRParsingException
-from robotoff.insights.question import QuestionFormatterFactory, QuestionFormatter
-from robotoff.ml.object_detection import ObjectDetectionModelRegistry
-from robotoff.ml.category.neural.model import (
-    ModelRegistry,
-    filter_blacklisted_categories,
+from robotoff.insights.extraction import (
+    DEFAULT_PREDICTION_TYPES,
+    extract_ocr_predictions,
 )
-from robotoff.models import ProductInsight, UserAnnotation
-from robotoff.off import http_session, OFFAuthentication
+from robotoff.insights.question import QuestionFormatter, QuestionFormatterFactory
+from robotoff.logos import generate_insights_from_annotated_logos
+from robotoff.models import (
+    ImageModel,
+    ImagePrediction,
+    LogoAnnotation,
+    ProductInsight,
+    batch_insert,
+)
+from robotoff.off import (
+    OFFAuthentication,
+    generate_image_path,
+    get_product,
+    get_server_type,
+)
+from robotoff.prediction.category.neural.category_classifier import CategoryClassifier
+from robotoff.prediction.object_detection import ObjectDetectionModelRegistry
+from robotoff.prediction.ocr.dataclass import OCRParsingException
+from robotoff.prediction.types import PredictionType
 from robotoff.products import get_product_dataset_etag
-from robotoff.utils import get_logger, get_image_from_url
+from robotoff.spellcheck import SPELLCHECKERS, Spellchecker
+from robotoff.taxonomy import TaxonomyType, get_taxonomy, match_unprefixed_value
+from robotoff.utils import get_image_from_url, get_logger, http_session
 from robotoff.utils.es import get_es_client
 from robotoff.utils.i18n import TranslationStore
+from robotoff.utils.text import get_tag
 from robotoff.utils.types import JSONType
 from robotoff.workers.client import send_ipc_event
 
-import sentry_sdk
-from sentry_sdk.integrations.falcon import FalconIntegration
-
 logger = get_logger()
 
-sentry_sdk.init(dsn=settings.SENTRY_DSN, integrations=[FalconIntegration()])
+settings.init_sentry(integrations=[FalconIntegration()])
 
 es_client = get_es_client()
 
 TRANSLATION_STORE = TranslationStore()
 TRANSLATION_STORE.load()
 
+###########
+# IMPORTANT: remember to update documentation at doc/references/api.md if you change API
+###########
+
 
 class ProductInsightResource:
     def on_get(self, req: falcon.Request, resp: falcon.Response, barcode: str):
@@ -77,7 +94,7 @@ class ProductInsightDetail:
         except ProductInsight.DoesNotExist:
             raise falcon.HTTPNotFound()
 
-        resp.media = insight.serialize(full=True)
+        resp.media = insight.to_dict()
 
 
 class InsightCollection:
@@ -117,9 +134,7 @@ class InsightCollection:
         )
 
         offset: int = (page - 1) * count
-        insights = [
-            i.serialize(full=True) for i in get_insights_(limit=count, offset=offset)
-        ]
+        insights = [i.to_dict() for i in get_insights_(limit=count, offset=offset)]
         response["count"] = get_insights_(count=True)
 
         if not insights:
@@ -133,31 +148,26 @@ class InsightCollection:
 
 
 class RandomInsightResource:
-    def on_get(self, req, resp):
+    def on_get(self, req: falcon.Request, resp: falcon.Response):
         insight_type: Optional[str] = req.get_param("type")
         country: Optional[str] = req.get_param("country")
         value_tag: Optional[str] = req.get_param("value_tag")
         server_domain: Optional[str] = req.get_param("server_domain")
+        count: int = req.get_param_as_int("count", default=1, min_value=1, max_value=50)
 
-        insights = list(
+        keep_types = [insight_type] if insight_type else None
+        insights: List[ProductInsight] = list(
             get_insights(
-                keep_types=[insight_type],
+                keep_types=keep_types,
                 country=country,
                 value_tag=value_tag,
                 order_by="random",
                 server_domain=server_domain,
-                limit=1,
+                limit=count,
             )
         )
 
-        response = {}
-        if not insights:
-            response["status"] = "no_insights"
-        else:
-            response["insight"] = insights[0].serialize()
-            response["status"] = "found"
-
-        resp.media = response
+        resp.media = {"insights": [insight.serialize() for insight in insights]}
 
 
 def parse_auth(req: falcon.Request) -> Optional[OFFAuthentication]:
@@ -173,7 +183,7 @@ def parse_auth(req: falcon.Request) -> Optional[OFFAuthentication]:
     if authorization is not None:
         try:
             username, password = basic_decode(authorization)
-        except BasicAuthDecodeError as e:
+        except BasicAuthDecodeError:
             raise falcon.HTTPUnauthorized(
                 "Invalid authentication, Basic auth expected."
             )
@@ -187,18 +197,26 @@ def parse_auth(req: falcon.Request) -> Optional[OFFAuthentication]:
 
 
 class AnnotateInsightResource:
-    def on_post(self, req, resp):
+    def on_post(self, req: falcon.Request, resp: falcon.Response):
         insight_id = req.get_param("insight_id", required=True)
         annotation = req.get_param_as_int(
             "annotation", required=True, min_value=-1, max_value=1
         )
 
         update = req.get_param_as_bool("update", default=True)
+        data = req.get_param_as_json("data")
 
         auth: Optional[OFFAuthentication] = parse_auth(req)
 
+        username = auth.get_username() if auth else "unknown annotator"
+        logger.info(
+            "New annotation received from {} (annotation: {}, insight: {})".format(
+                username, annotation, insight_id
+            )
+        )
+
         annotation_result = save_insight(
-            insight_id, annotation, update=update, auth=auth
+            insight_id, annotation, update=update, data=data, auth=auth
         )
 
         resp.media = {
@@ -208,27 +226,68 @@ class AnnotateInsightResource:
 
 
 class IngredientSpellcheckResource:
-    def on_post(self, req, resp):
-        text = req.get_param("text", required=True)
+    def on_get(self, req: falcon.Request, resp: falcon.Response):
+        self.spellcheck(req, resp)
 
-        corrections = generate_corrections(es_client, text, confidence=1)
-        term_corrections = list(
-            itertools.chain.from_iterable((c.term_corrections for c in corrections))
+    def on_post(self, req: falcon.Request, resp: falcon.Response):
+        self.spellcheck(req, resp)
+
+    def spellcheck(self, req: falcon.Request, resp: falcon.Response):
+        text = req.get_param("text")
+        if text is None:
+            barcode = req.get_param("barcode")
+            if barcode is None:
+                raise falcon.HTTPBadRequest("text or barcode is required.")
+
+            product = get_product(barcode) or {}
+            text = product.get("ingredients_text_fr")
+            if text is None:
+                resp.media = {"status": "not_found"}
+                return
+
+        index_name = req.get_param("index", default="product_all")
+        confidence = req.get_param_as_float("confidence", default=0.5)
+        pipeline = req.get_param_as_list("pipeline") or None
+        safe = req.get_param_as_bool("safe", blank_as_true=False)
+
+        if safe is not None and pipeline:
+            raise falcon.HTTPBadRequest(
+                "pipeline and safe parameters cannot be used together"
+            )
+
+        if pipeline:
+            for item in pipeline:
+                if item not in SPELLCHECKERS:
+                    raise falcon.HTTPBadRequest(f"unknown pipeline item: {item}")
+        elif safe:
+            pipeline = ["patterns", "percentages", "vocabulary"]
+
+        spellchecker = Spellchecker.load(
+            client=es_client,
+            pipeline=pipeline,
+            index_name=index_name,
+            confidence=confidence,
         )
+        correction_item = spellchecker.correct(text)
 
         resp.media = {
-            "corrections": [dataclasses.asdict(c) for c in corrections],
             "text": text,
-            "corrected": generate_corrected_text(term_corrections, text),
+            "corrected": correction_item.latest_correction,
+            "corrections": correction_item.corrections,
         }
 
 
 class NutrientPredictorResource:
-    def on_get(self, req, resp):
+    def on_get(self, req: falcon.Request, resp: falcon.Response):
         ocr_url = req.get_param("ocr_url", required=True)
 
+        if not ocr_url.endswith(".json"):
+            raise falcon.HTTPBadRequest("a JSON file is expected")
+
         try:
-            insights = extract_ocr_insights(ocr_url, [InsightType.nutrient.name])
+            product_predictions = extract_ocr_predictions(
+                ocr_url, [PredictionType.nutrient]
+            )
 
         except requests.exceptions.RequestException:
             resp.media = {
@@ -245,20 +304,21 @@ class NutrientPredictorResource:
             }
             return
 
-        if not insights:
+        if not product_predictions:
             resp.media = {
                 "nutrients": {},
             }
         else:
-            resp.media = {"nutrients": insights["nutrient"][0]["nutrients"]}
+            nutrient_predictions = product_predictions[PredictionType.nutrient]
+            resp.media = nutrient_predictions.to_dict()
 
 
 class OCRInsightsPredictorResource:
-    def on_get(self, req, resp):
+    def on_get(self, req: falcon.Request, resp: falcon.Response):
         ocr_url = req.get_param("ocr_url", required=True)
 
         try:
-            insights = extract_ocr_insights(ocr_url, DEFAULT_INSIGHT_TYPES)
+            insights = extract_ocr_predictions(ocr_url, DEFAULT_PREDICTION_TYPES)
 
         except requests.exceptions.RequestException:
             resp.media = {
@@ -281,75 +341,46 @@ class OCRInsightsPredictorResource:
 
 
 class CategoryPredictorResource:
-    def on_get(self, req, resp):
+    def on_get(self, req: falcon.Request, resp: falcon.Response):
         barcode = req.get_param("barcode", required=True)
         deepest_only = req.get_param_as_bool("deepest_only", default=False)
-        blacklist = req.get_param_as_bool("blacklist", default=False)
-        model = ModelRegistry.get()
-        predicted = model.predict_from_barcode(barcode, deepest_only=deepest_only)
 
-        if predicted:
-            if blacklist:
-                predicted = filter_blacklisted_categories(predicted)
+        categories = []
 
-            predicted = [
-                {"category": category, "confidence": confidence,}
-                for category, confidence in predicted
-            ]
+        product = get_product(barcode)
+        if product:
+            predicted = CategoryClassifier(
+                get_taxonomy(TaxonomyType.category.name)
+            ).predict(product, deepest_only)
 
-        predicted = predicted or []
-        resp.media = {"categories": predicted}
+            if predicted:
+                categories = [
+                    {
+                        "category": prediction.category,
+                        "confidence": prediction.confidence,
+                    }
+                    for prediction in predicted
+                ]
+
+        resp.media = {"categories": categories}
 
 
 class UpdateDatasetResource:
-    def on_post(self, req, resp):
+    def on_post(self, req: falcon.Request, resp: falcon.Response):
         send_ipc_event("download_dataset")
 
         resp.media = {
             "status": "scheduled",
         }
 
-    def on_get(self, req, resp):
+    def on_get(self, req: falcon.Request, resp: falcon.Response):
         resp.media = {
             "etag": get_product_dataset_etag(),
         }
 
 
-class InsightImporterResource:
-    def on_post(self, req, resp):
-        logger.info("New insight import request")
-        insight_type = req.get_param("type", required=True)
-        server_domain = req.get_param("server_domain", required=True)
-
-        if insight_type not in (t.name for t in InsightType):
-            raise falcon.HTTPBadRequest(
-                description="unknown insight type: " "'{}'".format(insight_type)
-            )
-
-        content = req.get_param("file", required=True)
-
-        logger.info("Insight type: '{}'".format(insight_type))
-
-        lines = [l for l in io.TextIOWrapper(content.file)]
-
-        send_ipc_event(
-            "import_insights",
-            {
-                "insight_type": insight_type,
-                "items": lines,
-                "server_domain": server_domain,
-            },
-        )
-
-        logger.info("Import scheduled")
-
-        resp.media = {
-            "status": "scheduled",
-        }
-
-
 class ImageImporterResource:
-    def on_post(self, req, resp):
+    def on_post(self, req: falcon.Request, resp: falcon.Response):
         barcode = req.get_param("barcode", required=True)
         image_url = req.get_param("image_url", required=True)
         ocr_url = req.get_param("ocr_url", required=True)
@@ -377,8 +408,102 @@ class ImageImporterResource:
         }
 
 
+class ImageCropResource:
+    def on_get(self, req: falcon.Request, resp: falcon.Response):
+        image_url = req.get_param("image_url", required=True)
+        y_min = req.get_param_as_float("y_min", required=True)
+        x_min = req.get_param_as_float("x_min", required=True)
+        y_max = req.get_param_as_float("y_max", required=True)
+        x_max = req.get_param_as_float("x_max", required=True)
+        image = get_image_from_url(image_url)
+
+        if image is None:
+            raise falcon.HTTPBadRequest("invalid image")
+
+        (left, right, top, bottom) = (
+            x_min * image.width,
+            x_max * image.width,
+            y_min * image.height,
+            y_max * image.height,
+        )
+        cropped_image = image.crop((left, top, right, bottom))
+        image_response(cropped_image, resp)
+
+
+class ImagePredictionImporterResource:
+    @jsonschema.validate(schema.IMAGE_PREDICTION_IMPORTER_SCHEMA)
+    def on_post(self, req: falcon.Request, resp: falcon.Response):
+        timestamp = datetime.datetime.utcnow()
+        inserts = []
+
+        for prediction in req.media["predictions"]:
+            server_domain: str = prediction.get(
+                "server_domain", settings.OFF_SERVER_DOMAIN
+            )
+            server_type: str = get_server_type(server_domain).name
+            source_image = generate_image_path(
+                prediction["barcode"], prediction.pop("image_id")
+            )
+            inserts.append(
+                {
+                    "timestamp": timestamp,
+                    "server_domain": server_domain,
+                    "server_type": server_type,
+                    "source_image": source_image,
+                    **prediction,
+                }
+            )
+
+        inserted = batch_insert(ImagePrediction, inserts)
+        logger.info("{} image predictions inserted".format(inserted))
+
+
+class ImagePredictionFetchResource:
+    def on_get(self, req: falcon.Request, resp: falcon.Response):
+        count: int = req.get_param_as_int("count", min_value=1, default=25)
+        model_name: Optional[str] = req.get_param("model_name")
+        type_: Optional[str] = req.get_param("type")
+        model_version: Optional[str] = req.get_param("model_version")
+        server_domain: Optional[str] = req.get_param("server_domain")
+        barcode: Optional[str] = req.get_param("barcode")
+        min_confidence: Optional[float] = req.get_param_as_float("min_confidence")
+        random: bool = req.get_param_as_bool("random", default=True)
+
+        where_clauses = []
+
+        if model_name is not None:
+            where_clauses.append(ImagePrediction.model_name == model_name)
+
+        if model_version is not None:
+            where_clauses.append(ImagePrediction.model_version == model_version)
+
+        if type_ is not None:
+            where_clauses.append(ImagePrediction.type == type_)
+
+        if server_domain:
+            where_clauses.append(ImageModel.server_domain == server_domain)
+
+        if min_confidence is not None:
+            where_clauses.append(ImagePrediction.max_confidence >= min_confidence)
+
+        if barcode is not None:
+            where_clauses.append(ImageModel.barcode == barcode)
+
+        query = ImagePrediction.select().join(ImageModel)
+
+        if where_clauses:
+            query = query.where(*where_clauses)
+
+        if random:
+            query = query.order_by(peewee.fn.Random())
+
+        query = query.limit(count)
+        items = [item.to_dict() for item in query.iterator()]
+        resp.media = {"predictions": items}
+
+
 class ImagePredictorResource:
-    def on_get(self, req, resp):
+    def on_get(self, req: falcon.Request, resp: falcon.Response):
         image_url = req.get_param("image_url", required=True)
         models: List[str] = req.get_param_as_list("models", required=True)
 
@@ -417,25 +542,228 @@ class ImagePredictorResource:
             result = model.detect_from_image(image, output_image=output_image)
 
             if output_image:
-                self.image_response(result.boxed_image, resp)
+                image_response(result.boxed_image, resp)
                 return
             else:
                 predictions[model_name] = result.to_json()
 
         resp.media = {"predictions": predictions}
 
-    @staticmethod
-    def image_response(image: Image.Image, resp: falcon.Response) -> None:
-        resp.content_type = "image/jpeg"
-        fp = io.BytesIO()
-        image.save(fp, "JPEG")
-        resp.stream_len = fp.tell()
-        fp.seek(0)
-        resp.stream = fp
+
+def image_response(image: Image.Image, resp: falcon.Response) -> None:
+    resp.content_type = "image/jpeg"
+    fp = io.BytesIO()
+    image.save(fp, "JPEG")
+    resp.stream_len = fp.tell()
+    fp.seek(0)
+    resp.stream = fp
+
+
+class ImageLogoResource:
+    def on_get(self, req: falcon.Request, resp: falcon.Response):
+        logo_ids: Optional[List[str]] = req.get_param_as_list(
+            "logo_ids", required=False
+        )
+
+        if logo_ids is not None:
+            self.fetch_logos(logo_ids, resp)
+        else:
+            self.search(req, resp)
+
+    def search(self, req: falcon.Request, resp: falcon.Response):
+        count: int = req.get_param_as_int(
+            "count", min_value=1, max_value=2000, default=25
+        )
+        type_: Optional[str] = req.get_param("type")
+        barcode: Optional[str] = req.get_param("barcode")
+        value: Optional[str] = req.get_param("value")
+        min_confidence: Optional[float] = req.get_param_as_float("min_confidence")
+        random: bool = req.get_param_as_bool("random", default=False)
+        server_domain: Optional[str] = req.get_param("server_domain")
+        annotated: bool = req.get_param_as_bool("annotated", default=False)
+
+        where_clauses = [LogoAnnotation.annotation_value.is_null(not annotated)]
+        join_image_prediction = False
+        join_image_model = False
+
+        if server_domain:
+            where_clauses.append(ImageModel.server_domain == server_domain)
+            join_image_model = True
+
+        if min_confidence is not None:
+            where_clauses.append(ImagePrediction.max_confidence >= min_confidence)
+            join_image_prediction = True
+
+        if barcode is not None:
+            where_clauses.append(ImageModel.barcode == barcode)
+            join_image_model = True
+
+        if type_ is not None:
+            where_clauses.append(LogoAnnotation.annotation_type == type_)
+
+        if value is not None:
+            value_tag = get_tag(value)
+            where_clauses.append(LogoAnnotation.annotation_value_tag == value_tag)
+
+        query = LogoAnnotation.select()
+        join_image_prediction = join_image_prediction or join_image_model
+
+        if join_image_prediction:
+            query = query.join(ImagePrediction)
+
+            if join_image_model:
+                query = query.join(ImageModel)
+
+        if where_clauses:
+            query = query.where(*where_clauses)
+
+        query_count = query.count()
+
+        if random:
+            query = query.order_by(peewee.fn.Random())
+
+        query = query.limit(count)
+        items = [item.to_dict() for item in query.iterator()]
+
+        for item in items:
+            image_prediction = item.pop("image_prediction")
+            item["image"] = image_prediction["image"]
+
+        resp.media = {"logos": items, "count": query_count}
+
+    def fetch_logos(self, logo_ids: List[str], resp: falcon.Response):
+        logos = []
+        for logo in (
+            LogoAnnotation.select()
+            .join(ImagePrediction)
+            .join(ImageModel)
+            .where(LogoAnnotation.id.in_(logo_ids))
+            .iterator()
+        ):
+            logo_dict = logo.to_dict()
+            image_prediction = logo_dict.pop("image_prediction")
+            logo_dict["image"] = image_prediction["image"]
+            logos.append(logo_dict)
+
+        resp.media = {"logos": logos, "count": len(logos)}
+
+
+class ImageLogoDetailResource:
+    def on_get(self, req: falcon.Request, resp: falcon.Response, logo_id: int):
+        logo = LogoAnnotation.get_or_none(id=logo_id)
+
+        if logo is None:
+            resp.status = falcon.HTTP_404
+            return
+
+        logo_dict = logo.to_dict()
+        image_prediction = logo_dict.pop("image_prediction")
+        logo_dict["image"] = image_prediction["image"]
+        resp.media = logo_dict
+
+    @jsonschema.validate(schema.UPDATE_LOGO_SCHEMA)
+    def on_put(self, req: falcon.Request, resp: falcon.Response, logo_id: int):
+        logo = LogoAnnotation.get_or_none(id=logo_id)
+
+        if logo is None:
+            resp.status = falcon.HTTP_404
+            return
+
+        type_ = req.media["type"]
+        value = req.media["value"] or None
+        updated = False
+
+        if type_ != logo.annotation_type:
+            logo.annotation_type = type_
+            updated = True
+
+        if value != logo.annotation_value:
+            logo.annotation_value = value
+
+            if value is not None:
+                value_tag = get_tag(value)
+                logo.annotation_value_tag = value_tag
+                logo.taxonomy_value = match_unprefixed_value(value_tag, type_)
+            else:
+                logo.annotation_value_tag = None
+                logo.taxonomy_value = None
+
+            updated = True
+
+        if updated:
+            auth = parse_auth(req)
+            username = None if auth is None else auth.get_username()
+            logo.username = username
+            logo.completed_at = datetime.datetime.utcnow()
+            logo.save()
+
+        resp.status = falcon.HTTP_204
+
+
+class ImageLogoAnnotateResource:
+    def on_post(self, req: falcon.Request, resp: falcon.Response):
+        server_domain = req.media.get("server_domain", settings.OFF_SERVER_DOMAIN)
+        annotations = req.media["annotations"]
+        auth = parse_auth(req)
+        username = None if auth is None else auth.get_username()
+        completed_at = datetime.datetime.utcnow()
+        annotated_logos = []
+
+        for annotation in annotations:
+            logo_id = annotation["logo_id"]
+            type_ = annotation["type"]
+            value = annotation["value"] or None
+            logo = LogoAnnotation.get_by_id(logo_id)
+
+            if value is not None:
+                logo.annotation_value = value
+                value_tag = get_tag(value)
+                logo.annotation_value_tag = value_tag
+                logo.taxonomy_value = match_unprefixed_value(value_tag, type_)
+
+            logo.annotation_type = type_
+            logo.username = username
+            logo.completed_at = completed_at
+            logo.save()
+            annotated_logos.append(logo)
+
+        generate_insights_from_annotated_logos(annotated_logos, server_domain)
+
+
+class ImageLogoUpdateResource:
+    def on_post(self, req: falcon.Request, resp: falcon.Response):
+        source_value = req.get_param("source_value", required=True)
+        source_type = req.get_param("source_type", required=True)
+        target_value = req.get_param("target_value", required=True)
+        target_type = req.get_param("target_type", required=True)
+
+        auth = parse_auth(req)
+        username = None if auth is None else auth.get_username()
+        completed_at = datetime.datetime.utcnow()
+
+        target_value_tag = get_tag(target_value)
+        source_value_tag = get_tag(source_value)
+        taxonomy_value = match_unprefixed_value(target_value_tag, target_type)
+
+        query = LogoAnnotation.update(
+            {
+                LogoAnnotation.annotation_type: target_type,
+                LogoAnnotation.annotation_value: target_value,
+                LogoAnnotation.annotation_value_tag: target_value_tag,
+                LogoAnnotation.taxonomy_value: taxonomy_value,
+                LogoAnnotation.username: username,
+                LogoAnnotation.completed_at: completed_at,
+            }
+        ).where(
+            LogoAnnotation.annotation_type == source_type,
+            LogoAnnotation.annotation_value_tag == source_value_tag,
+        )
+        updated = query.execute()
+        resp.media = {"updated": updated}
 
 
 class WebhookProductResource:
-    def on_post(self, req, resp):
+    def on_post(self, req: falcon.Request, resp: falcon.Response):
         barcode = req.get_param("barcode", required=True)
         action = req.get_param("action", required=True)
         server_domain = req.get_param("server_domain", required=True)
@@ -460,12 +788,12 @@ class WebhookProductResource:
 
         if action == "updated":
             send_ipc_event(
-                "product_updated", {"barcode": barcode, "server_domain": server_domain,}
+                "product_updated", {"barcode": barcode, "server_domain": server_domain}
             )
 
         elif action == "deleted":
             send_ipc_event(
-                "product_deleted", {"barcode": barcode, "server_domain": server_domain,}
+                "product_deleted", {"barcode": barcode, "server_domain": server_domain}
             )
 
         resp.media = {
@@ -474,13 +802,18 @@ class WebhookProductResource:
 
 
 class ProductQuestionsResource:
-    def on_get(self, req, resp, barcode):
-        response = {}
+    """Get a question about a product to confirm/infirm an insight
+
+    see also doc/explanation/questions.md
+    """
+
+    def on_get(self, req: falcon.Request, resp: falcon.Response, barcode: str):
+        response: JSONType = {}
         count: int = req.get_param_as_int("count", min_value=1) or 1
         lang: str = req.get_param("lang", default="en")
         server_domain: Optional[str] = req.get_param("server_domain")
 
-        keep_types = QuestionFormatterFactory.get_available_types()
+        keep_types = QuestionFormatterFactory.get_default_types()
         insights = list(
             get_insights(
                 barcode=barcode,
@@ -540,7 +873,7 @@ def get_questions_resource_on_get(
         reserved_barcode = None
 
     if keep_types is None:
-        keep_types = QuestionFormatterFactory.get_available_types()
+        keep_types = QuestionFormatterFactory.get_default_types()
     else:
         # Limit the number of types to prevent slow SQL queries
         keep_types = keep_types[:10]
@@ -562,7 +895,7 @@ def get_questions_resource_on_get(
 
     insights = list(get_insights_(limit=count))
     response["count"] = get_insights_(count=True)
-
+    # This code should be merged with the one in ProductQuestionsResource.get
     if not insights:
         response["questions"] = []
         response["status"] = "no_questions"
@@ -586,12 +919,25 @@ def get_questions_resource_on_get(
 
 
 class StatusResource:
-    def on_get(self, req, resp):
+    def on_get(self, req: falcon.Request, resp: falcon.Response):
         resp.media = {
             "status": "running",
         }
 
 
+class HealthResource:
+    def on_get(self, req: falcon.Request, resp: falcon.Response):
+        from robotoff.health import health
+
+        message, status, headers = health.run()
+        resp.media = {
+            "message": orjson.loads(message),
+            "status": status,
+            "headers": headers,
+        }
+        resp.status = str(status)
+
+
 class DumpResource:
     def on_get(self, req: falcon.Request, resp: falcon.Response):
         keep_types: Optional[List[str]] = req.get_param_as_list(
@@ -617,7 +963,7 @@ class DumpResource:
 
         with tempfile.TemporaryFile("w+", newline="") as temp_f:
             for insight in insights_iter:
-                serial = insight.serialize(full=True)
+                serial = orjson.loads(orjson.dumps(insight.to_dict()))
 
                 if writer is None:
                     writer = csv.DictWriter(temp_f, fieldnames=serial.keys())
@@ -638,7 +984,7 @@ class DumpResource:
 class UserStatisticsResource:
     def on_get(self, req: falcon.Request, resp: falcon.Response, username: str):
         annotation_count = (
-            UserAnnotation.select().where(UserAnnotation.username == username).count()
+            ProductInsight.select().where(ProductInsight.username == username).count()
         )
         resp.media = {"count": {"annotations": annotation_count}}
 
@@ -654,16 +1000,24 @@ cors = CORS(
 api = falcon.API(
     middleware=[cors.middleware, MultipartMiddleware(), DBConnectionMiddleware()]
 )
+
+json_handler = falcon.media.JSONHandler(dumps=orjson.dumps, loads=orjson.loads)
+extra_handlers = {
+    "application/json": json_handler,
+}
+
+api.resp_options.media_handlers.update(extra_handlers)
+
 # Parse form parameters
 api.req_options.auto_parse_form_urlencoded = True
 api.req_options.strip_url_path_trailing_slash = True
 api.req_options.auto_parse_qs_csv = True
+# defines urls
 api.add_route("/api/v1/insights/{barcode}", ProductInsightResource())
 api.add_route("/api/v1/insights/detail/{insight_id:uuid}", ProductInsightDetail())
 api.add_route("/api/v1/insights", InsightCollection())
 api.add_route("/api/v1/insights/random", RandomInsightResource())
 api.add_route("/api/v1/insights/annotate", AnnotateInsightResource())
-api.add_route("/api/v1/insights/import", InsightImporterResource())
 api.add_route("/api/v1/predict/ingredients/spellcheck", IngredientSpellcheckResource())
 api.add_route("/api/v1/predict/nutrient", NutrientPredictorResource())
 api.add_route("/api/v1/predict/ocr_insights", OCRInsightsPredictorResource())
@@ -671,10 +1025,18 @@ api.add_route("/api/v1/predict/category", CategoryPredictorResource())
 api.add_route("/api/v1/products/dataset", UpdateDatasetResource())
 api.add_route("/api/v1/webhook/product", WebhookProductResource())
 api.add_route("/api/v1/images/import", ImageImporterResource())
+api.add_route("/api/v1/images/crop", ImageCropResource())
+api.add_route("/api/v1/images/predictions/import", ImagePredictionImporterResource())
+api.add_route("/api/v1/images/predictions", ImagePredictionFetchResource())
 api.add_route("/api/v1/images/predict", ImagePredictorResource())
+api.add_route("/api/v1/images/logos", ImageLogoResource())
+api.add_route("/api/v1/images/logos/{logo_id:int}", ImageLogoDetailResource())
+api.add_route("/api/v1/images/logos/annotate", ImageLogoAnnotateResource())
+api.add_route("/api/v1/images/logos/update", ImageLogoUpdateResource())
 api.add_route("/api/v1/questions/{barcode}", ProductQuestionsResource())
 api.add_route("/api/v1/questions/random", RandomQuestionsResource())
 api.add_route("/api/v1/questions/popular", PopularQuestionsResource())
 api.add_route("/api/v1/status", StatusResource())
+api.add_route("/api/v1/health", HealthResource())
 api.add_route("/api/v1/dump", DumpResource())
 api.add_route("/api/v1/users/statistics/{username}", UserStatisticsResource())
