@@ -1,19 +1,18 @@
-from typing import Union, Optional, List, Iterable
+from typing import Dict, Iterable, List, Optional, Union
+
+import peewee
 
 from robotoff import settings
 from robotoff.insights.annotate import (
-    InsightAnnotatorFactory,
-    AnnotationResult,
     ALREADY_ANNOTATED_RESULT,
     UNKNOWN_INSIGHT_RESULT,
+    AnnotationResult,
+    InsightAnnotatorFactory,
 )
 from robotoff.models import ProductInsight
 from robotoff.off import OFFAuthentication
 from robotoff.utils import get_logger
 
-import peewee
-
-
 logger = get_logger(__name__)
 
 
@@ -32,12 +31,16 @@ def get_insights(
     limit: Optional[int] = 25,
     offset: Optional[int] = None,
     count: bool = False,
+    latent: Optional[bool] = False,
 ) -> Iterable[ProductInsight]:
     if server_domain is None:
         server_domain = settings.OFF_SERVER_DOMAIN
 
     where_clauses = [ProductInsight.server_domain == server_domain]
 
+    if latent is not None:
+        where_clauses.append(ProductInsight.latent == latent)
+
     if annotated is not None:
         where_clauses.append(ProductInsight.annotation.is_null(not annotated))
 
@@ -90,6 +93,7 @@ def save_insight(
     insight_id: str,
     annotation: int,
     update: bool = True,
+    data: Optional[Dict] = None,
     auth: Optional[OFFAuthentication] = None,
 ) -> AnnotationResult:
     try:
@@ -104,4 +108,4 @@ def save_insight(
         return ALREADY_ANNOTATED_RESULT
 
     annotator = InsightAnnotatorFactory.get(insight.type)
-    return annotator.annotate(insight, annotation, update, auth=auth)
+    return annotator.annotate(insight, annotation, update, data=data, auth=auth)
