@@ -1,10 +1,10 @@
 import datetime
 import json
-from urllib.parse import urlparse
 from typing import List, Optional
+from urllib.parse import urlparse
 
-from influxdb import InfluxDBClient
 import requests
+from influxdb import InfluxDBClient
 
 from robotoff import settings
 from robotoff.utils import get_logger
@@ -58,10 +58,13 @@ COUNTRY_TAGS = [
     "no",
     "in",
     "tn",
+    "dk",
 ]
 
 
 def get_influx_client() -> InfluxDBClient:
+    if not settings.INFLUXDB_HOST:
+        return None
     return InfluxDBClient(
         settings.INFLUXDB_HOST,
         settings.INFLUXDB_PORT,
@@ -73,14 +76,12 @@ def get_influx_client() -> InfluxDBClient:
 
 def get_product_count(country_tag: str) -> int:
     r = requests.get(
-        "https://{}.openfoodfacts.org/3.json?fields=null".format(country_tag)
+        settings.BaseURLProvider().country(country_tag).get() + "/3.json?fields=null"
     ).json()
-    return r["count"]
+    return int(r["count"])
 
 
 def save_facet_metrics():
-    client = get_influx_client()
-
     inserts = []
     target_datetime = datetime.datetime.now()
     product_counts = {
@@ -106,7 +107,9 @@ def save_facet_metrics():
         )
 
     inserts += generate_metrics_from_path("world", "/countries?json=1", target_datetime)
-    client.write_points(inserts)
+    client = get_influx_client()
+    if client is not None:
+        client.write_points(inserts)
 
 
 def get_facet_name(url: str) -> str:
@@ -121,7 +124,7 @@ def generate_metrics_from_path(
     facet: Optional[str] = None,
 ) -> List:
     inserts: List = []
-    url = f"https://{country_tag}-en.openfoodfacts.org{path}"
+    url = settings.BaseURLProvider().country(country_tag + "-en").get() + path
 
     if facet is None:
         facet = get_facet_name(url)
