@@ -1,11 +1,13 @@
+"""Interacting with OFF server to eg. update products or get infos
+"""
 import enum
 import re
-from typing import List, Dict, Optional, Tuple, Union
-
-import requests
+from typing import Dict, List, Optional, Tuple, Union
 
 from robotoff import settings
-from robotoff.utils import get_logger
+from robotoff.utils import get_logger, http_session
+
+logger = get_logger(__name__)
 
 
 class OFFAuthentication:
@@ -24,6 +26,33 @@ class OFFAuthentication:
         self.username = username
         self.password = password
 
+    def get_username(self) -> Optional[str]:
+        if self.username is not None:
+            return self.username
+
+        elif self.session_cookie is not None:
+            splitted = self.session_cookie.split("&")
+
+            if splitted:
+                is_next = False
+                for split in splitted:
+                    if split == "user_id":
+                        is_next = True
+                        continue
+                    elif is_next:
+                        if split:
+                            return split
+                        else:
+                            break
+
+            logger.warning(
+                "Unable to extract username from session cookie: {}".format(
+                    self.session_cookie
+                )
+            )
+
+        return None
+
 
 class ServerType(enum.Enum):
     off = 1
@@ -32,38 +61,32 @@ class ServerType(enum.Enum):
     opf = 4
 
 
-http_session = requests.Session()
-USER_AGENT_HEADERS = {
-    "User-Agent": settings.ROBOTOFF_USER_AGENT,
-}
-http_session.headers.update(USER_AGENT_HEADERS)
-
-AUTH = ("roboto-app", settings.OFF_PASSWORD)
-AUTH_DICT = {
-    "user_id": AUTH[0],
-    "password": AUTH[1],
-}
-
 API_URLS: Dict[ServerType, str] = {
-    ServerType.off: "https://world.openfoodfacts.org",
+    ServerType.off: settings.BaseURLProvider().get(),
     ServerType.obf: "https://world.openbeautyfacts.org",
     ServerType.opf: "https://world.openproductfacts.org",
     ServerType.opff: "https://world.openpetfoodfacts.org",
 }
 
-logger = get_logger(__name__)
-
 
 BARCODE_PATH_REGEX = re.compile(r"^(...)(...)(...)(.*)$")
 
 
-def get_product_update_url(server_domain: str) -> str:
-    server_domain = server_domain.replace("api", "world")
-    return "https://{}/cgi/product_jqm2.pl".format(server_domain)
+def get_product_update_url(server: Union[ServerType, str]) -> str:
+    return "{}/cgi/product_jqm2.pl".format(get_base_url(server))
+
+
+def get_product_image_select_url(server: Union[ServerType, str]) -> str:
+    return "{}/cgi/product_image_crop.pl".format(get_base_url(server))
+
+
+def get_api_product_url(server: Union[ServerType, str]) -> str:
+    return "{}/api/v0/product".format(get_base_url(server))
 
 
 def get_base_url(server: Union[ServerType, str]) -> str:
     if isinstance(server, str):
+        server = server.replace("api", "world")
         return "https://{}".format(server)
     else:
         if server not in API_URLS:
@@ -72,11 +95,7 @@ def get_base_url(server: Union[ServerType, str]) -> str:
         return API_URLS[server]
 
 
-def get_api_product_url(server: Union[ServerType, str]) -> str:
-    return "{}/api/v0/product".format(get_base_url(server))
-
-
-def get_server_type(server_domain: str) -> Optional[ServerType]:
+def get_server_type(server_domain: str) -> ServerType:
     """Return the server type (off, obf, opff, opf) associated with the server
     domain, or None if the server_domain was not recognized."""
     server_split = server_domain.split(".")
@@ -93,8 +112,7 @@ def get_server_type(server_domain: str) -> Optional[ServerType]:
         elif domain == "openproductsfacts":
             return ServerType.opf
 
-    logger.warning("unknown server domain: {}".format(server_domain))
-    return None
+    raise ValueError("unknown server domain: {}".format(server_domain))
 
 
 def split_barcode(barcode: str) -> List[str]:
@@ -109,16 +127,22 @@ def split_barcode(barcode: str) -> List[str]:
     return [barcode]
 
 
-def generate_json_ocr_url(barcode: str, image_name: str) -> str:
+def generate_image_path(barcode: str, image_id: str) -> str:
     splitted_barcode = split_barcode(barcode)
-    path = "/{}/{}.json".format("/".join(splitted_barcode), image_name)
-    return settings.OFF_IMAGE_BASE_URL + path
+    return "/{}/{}.jpg".format("/".join(splitted_barcode), image_id)
 
 
-def generate_image_url(barcode: str, image_name: str) -> str:
+def generate_json_path(barcode: str, image_id: str) -> str:
     splitted_barcode = split_barcode(barcode)
-    path = "/{}/{}.jpg".format("/".join(splitted_barcode), image_name)
-    return settings.OFF_IMAGE_BASE_URL + path
+    return "/{}/{}.json".format("/".join(splitted_barcode), image_id)
+
+
+def generate_json_ocr_url(barcode: str, image_id: str) -> str:
+    return settings.OFF_IMAGE_BASE_URL + generate_json_path(barcode, image_id)
+
+
+def generate_image_url(barcode: str, image_id: str) -> str:
+    return settings.OFF_IMAGE_BASE_URL + generate_image_path(barcode, image_id)
 
 
 def is_valid_image(barcode: str, image_id: str) -> bool:
@@ -136,6 +160,7 @@ def get_product(
     barcode: str,
     fields: List[str] = None,
     server: Optional[Union[ServerType, str]] = None,
+    timeout: Optional[int] = 10,
 ) -> Optional[Dict]:
     fields = fields or []
 
@@ -150,7 +175,7 @@ def get_product(
         # See https://github.com/openfoodfacts/openfoodfacts-server/issues/1607
         url += "?fields={}".format(",".join(fields))
 
-    r = http_session.get(url)
+    r = http_session.get(url, timeout=timeout)
 
     if r.status_code != 200:
         return None
@@ -319,6 +344,7 @@ def update_product(
     params: Dict,
     server_domain: Optional[str] = None,
     auth: Optional[OFFAuthentication] = None,
+    timeout: Optional[int] = 15,
 ):
     if server_domain is None:
         server_domain = settings.OFF_SERVER_DOMAIN
@@ -337,17 +363,24 @@ def update_product(
             params["user_id"] = auth.username
             params["password"] = auth.password
     else:
-        params.update(AUTH_DICT)
+        params.update(settings.off_credentials())
 
         if comment:
             params["comment"] = comment + " (automated edit)"
 
+    if cookies is None and not params.get("password"):
+        raise ValueError(
+            "a password or a session cookie is required to update a product"
+        )
+
     request_auth: Optional[Tuple[str, str]] = None
     if server_domain.endswith("openfoodfacts.net"):
         # dev environment requires authentication
         request_auth = ("off", "off")
 
-    r = http_session.get(url, params=params, auth=request_auth, cookies=cookies)
+    r = http_session.get(
+        url, params=params, auth=request_auth, cookies=cookies, timeout=timeout
+    )
 
     r.raise_for_status()
     json = r.json()
@@ -358,17 +391,74 @@ def update_product(
         logger.warn("Unexpected status during product update: {}".format(status))
 
 
-def move_to(barcode: str, to: ServerType) -> bool:
+def move_to(barcode: str, to: ServerType, timeout: Optional[int] = 10) -> bool:
     if get_product(barcode, server=to) is not None:
         return False
 
-    url = "{}/cgi/product_jqm.pl".format(settings.OFF_BASE_WEBSITE_URL)
+    url = "{}/cgi/product_jqm.pl".format(settings.BaseURLProvider().get())
     params = {
         "type": "edit",
         "code": barcode,
         "new_code": to,
-        **AUTH_DICT,
+        **settings.off_credentials(),
     }
-    r = http_session.get(url, params=params)
+    r = http_session.get(url, params=params, timeout=timeout)
     data = r.json()
     return data["status"] == 1
+
+
+def select_rotate_image(
+    barcode: str,
+    image_id: str,
+    image_key: Optional[str] = None,
+    rotate: Optional[int] = None,
+    server_domain: Optional[str] = None,
+    auth: Optional[OFFAuthentication] = None,
+    timeout: Optional[int] = 15,
+):
+    if server_domain is None:
+        server_domain = settings.OFF_SERVER_DOMAIN
+
+    url = get_product_image_select_url(server_domain)
+    cookies = None
+    params = {
+        "code": barcode,
+        "imgid": image_id,
+    }
+
+    if rotate is not None:
+        if rotate not in (90, 180, 270):
+            raise ValueError("invalid value for rotation angle: {}".format(rotate))
+
+        params["angle"] = str(rotate)
+
+    if image_key is not None:
+        params["id"] = image_key
+
+    if auth is not None:
+        if auth.session_cookie:
+            cookies = {
+                "session": auth.session_cookie,
+            }
+        elif auth.username and auth.password:
+            params["user_id"] = auth.username
+            params["password"] = auth.password
+    else:
+        params.update(settings.off_credentials())
+
+    if cookies is None and not params.get("password"):
+        raise ValueError(
+            "a password or a session cookie is required to select an image"
+        )
+
+    request_auth: Optional[Tuple[str, str]] = None
+    if server_domain.endswith("openfoodfacts.net"):
+        # dev environment requires authentication
+        request_auth = ("off", "off")
+
+    r = http_session.post(
+        url, data=params, auth=request_auth, cookies=cookies, timeout=timeout
+    )
+
+    r.raise_for_status()
+    return r
